==> flutter_tide-0.6.1.tar.gz: lib/src/persisted_provider.dart <==
import 'package:flutter/widgets.dart';
import 'package:provider/provider.dart';
import 'package:tide/tide.dart';

import '../flutter_tide.dart';
import 'helpers/debounce.dart';

class PersistedStoreProvider<State> extends StatefulWidget {
  /// Creates a [StoreProvider] instance and exposes both the [Store]
  /// and its [Store.state] using `provider`.
  ///
  /// **DON'T** use this with an existing [Store] instance, as removing
  /// the provider from the widget tree will dispose the [Store].\
  /// Instead consider using [StateNotifierBuilder].
  ///
  /// `create` cannot be `null`.
  const PersistedStoreProvider({
    Key key,
    @required this.createStore,
    @required this.persistence,
    @required this.restoredbuilder,
    @required this.restoringBuilder,
    this.debouncing = const Duration(seconds: 1),
    this.lazy,
  })  : this.state = null,
        assert(createStore != null),
        super(
          key: key,
        );

  final Duration debouncing;
  final Persistence<State> persistence;
  final Create<Store<State>> createStore;
  final bool lazy;
  final TransitionBuilder restoredbuilder;
  final TransitionBuilder restoringBuilder;
  final Store<State> state;

  @override
  _PersistedStoreProviderState<State> createState() =>
      _PersistedStoreProviderState<State>();
}

class _PersistedStoreProviderState<S> extends State<PersistedStoreProvider<S>> {
  Future<Store<S>> future;
  Store<S> store;

  @override
  void initState() {
    future = _restoreState();
    super.initState();
  }

  Future<Store<S>> _restoreState() async {
    try {
      final result = await widget.persistence.read();
      store = Store(initialState: result) ?? widget.createStore(context);
    } catch (e) {
      store = widget.createStore(context);
    }

    store.addListener(_onStateChanged);

    return store;
  }

  void _onStateChanged(S state) async {
    if (widget.debouncing != null) {
      debounce('write_state', widget.debouncing, () async {
        await widget.persistence.write(state);
      });
    } else {
      await widget.persistence.write(state);
    }
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder<Store<S>>(
      future: future,
      builder: (context, snapshot) {
        if (snapshot.connectionState == ConnectionState.done) {
          return StoreProvider<S>.state(
            state: snapshot.data,
            builder: widget.restoredbuilder,
          );
        }

        return widget.restoringBuilder(context, null);
      },
    );
  }
}

==> flutter_tide-0.6.2.tar.gz: lib/src/persisted_provider.dart <==
import 'package:flutter/widgets.dart';
import 'package:provider/provider.dart';
import 'package:tide/tide.dart';

import '../flutter_tide.dart';
import 'helpers/debounce.dart';

class PersistedStoreProvider<State> extends StatefulWidget {
  /// Creates a [StoreProvider] instance and exposes both the [Store]
  /// and its [Store.state] using `provider`.
  ///
  /// **DON'T** use this with an existing [Store] instance, as removing
  /// the provider from the widget tree will dispose the [Store].\
  /// Instead consider using [StateNotifierBuilder].
  ///
  /// `create` cannot be `null`.
  const PersistedStoreProvider({
    Key key,
    @required this.createStore,
    @required this.persistence,
    @required this.restoredbuilder,
    @required this.restoringBuilder,
    this.debouncing = const Duration(seconds: 1),
    this.lazy,
  })  : this.state = null,
        assert(createStore != null),
        super(
          key: key,
        );

  final Duration debouncing;
  final Persistence<State> persistence;
  final Create<Store<State>> createStore;
  final bool lazy;
  final TransitionBuilder restoredbuilder;
  final TransitionBuilder restoringBuilder;
  final Store<State> state;

  @override
  _PersistedStoreProviderState<State> createState() =>
      _PersistedStoreProviderState<State>();
}

class _PersistedStoreProviderState<S> extends State<PersistedStoreProvider<S>> {
  Future<Store<S>> future;
  Store<S> store;

  @override
  void initState() {
    future = _restoreState();
    super.initState();
  }

  Future<Store<S>> _restoreState() async {
    try {
      final result = await widget.persistence.read();
      store = result != null
          ? Store(initialState: result)
          : widget.createStore(context);
    } catch (e) {
      store = widget.createStore(context);
    }

    store.addListener(_onStateChanged);

    return store;
  }

  void _onStateChanged(S state) async {
    if (widget.debouncing != null) {
      debounce('write_state', widget.debouncing, () async {
        await widget.persistence.write(state);
      });
    } else {
      await widget.persistence.write(state);
    }
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder<Store<S>>(
      future: future,
      builder: (context, snapshot) {
        if (snapshot.connectionState == ConnectionState.done) {
          return StoreProvider<S>.state(
            state: snapshot.data,
            builder: widget.restoredbuilder,
          );
        }

        return widget.restoringBuilder(context, null);
      },
    );
  }
}
